import acm.program.*;
import acm.util.*;
import java.io.*;

public class MusicPlayer extends ConsoleProgram {
	/* Frequencies of the notes in a scale. */
	private static final double[] FREQUENCIES = {
		220.000,  // A
		246.941,  // B
		261.626,  // C
		293.665,  // D
		329.628,  // E
		349.228,  // F
		391.995,  // G
	};
	
	/**
	 * Given a note name, returns the frequency associated with that note.
	 * The note should either have a form "[note][octave]" (for example,
	 * C2 or B1) or "rest."
	 * 
	 * @param note The note to play.
	 * @return Its frequency.
	 */
	private double frequencyForNote(String note) {
		/* Special case rests by playing a sound with extraordinarily low frequency. */
		if (note.equals("rest")) return 0.5;
		
		/* Otherwise, get the base frequency and multiply in the octave. */
		return baseNoteFrequency(note) * octaveMultiplier(note);
	}
	
	/**
	 * Given a note to play, returns its base frequency.
	 * 
	 * @param note The note to play.
	 * @return Its base frequency.
	 */
	private double baseNoteFrequency(String note) {
		return FREQUENCIES[note.charAt(0) - 'A'];
	}
	
	/**
	 * Given a note to play in the standard format, returns the multiplier
	 * associated with the octave. Each increase in octave doubles the
	 * frequency, so this multiplier is 2^(octave - 1).
	 * 
	 * @param note The note, in standard representation.
	 * @return The multiplier associated with the octave.
	 */
	private double octaveMultiplier(String note) {
		/* Since the note name is [note][octave], we can extract the octave by
		 * looking at everything but the first character as a string.
		 */
		return Math.pow(2, Integer.parseInt(note.substring(1)));
	}
	
	/**
	 * Creates a sine wave (pure tone) of the given frequency and
	 * length.
	 *
	 * @param frequency The frequence (in Hertz) of the sound wave.
	 * @param length The length (in seconds) of that sound wave.
	 * @return A sine wave of the given frequency and length.
	 */
	private double[] sineWave(double frequency, double length) {
		int numSamples = (int)(StdAudio.SAMPLE_RATE * length);
		double[] result = new double[numSamples];
		
		double wavelength = StdAudio.SAMPLE_RATE / frequency;
		
		for (int i = 0; i < result.length; i++) {
			double wavePosition = i / wavelength;
			result[i] = Math.sin(wavePosition * Math.PI * 2.0);
		}
		
		return result;
	}
	
	public void run() {
		try {
			/* Open the music file. */
			BufferedReader br = new BufferedReader(new FileReader("music2.txt"));
			
			while (true) {
				/* Read the note and duration. */
				String note     = br.readLine();
				String duration = br.readLine();
				if (note == null || duration == null) break;
				
				/* Play the note! */
				StdAudio.play(sineWave(frequencyForNote(note), Double.parseDouble(duration)));
			}
			
			br.close();
		} catch (IOException e) {
			println("No music for you!");
		}
	}
}
